	SUBROUTINE generate_answers(ok)

*	This procedure generates the alternatives
*	to be proposed to the student.
*	(COMPITO package)

	IMPLICIT NONE
	SAVE

	INCLUDE 'compito.inc'

	INTEGER ique, ialt, j		! loop indices
	INTEGER natt			! attempt counter
	LOGICAL found			! flag for finding a good alternative
	REAL rel1, rel2			! (fractional) differences
	REAL attempt			! tentative alternative to solution
	REAL alt_low, alt_up		! alternative range edges
        REAL ul,ll,exponent             ! range for order of mag. of
                                        ! alternative answers
	REAL rndm_sign			! random sign
	LOGICAL ok			! control interaction between answers and forcings
	INTEGER i0, i1			! limits of forced answers
        INTEGER ia1,ia2			! Auxiliary integer variables
	
*	external function
	REAL round			! round to significant digits
	REAL ran1


*	Executable code

* 	Zero the extrema
      i0 = 1
      i1 = 1
*	Set the variable ok
      ok = .FALSE.
      
      
	  DO 50000 ique = 1, n_questions		! loop on questions
* Added to account for literal answers
              IF (.NOT.lit_ans(ique))THEN            
*	Generate the forced answers
		CALL FORCED_ANSWERS(ique, i1)
c                print *,'forced',ique,i1		
* 	Check the distance between solutions and forced answers
        do ialt = i0, i1-1
           rel1 = ABS (solution(ique)-forced_a(ialt))
           rel2 = ABS (solution(ique)+forced_a(ialt))
           if(rel1 .LT. gap*rel2) return
        enddo		

*	  set up alternative range
          if(solution(ique).ne.0.)then
c		  alt_low = solution(ique) * scale_down
c		  alt_up  = solution(ique) * scale_up
C Added by isidoro.ferrante@pi.infn.it 28/3/2000
C to generate within a better range
                  ul=exprange*ran1(iseed)
                  ll=ul-exprange
	    else
                  ialt = 1
                  alt_low = forced_a(i1-1) * scale_down
                  alt_up  = forced_a(i1-1) * scale_up
	  endif
	  
            

	    DO 40000 ialt = 1, n_alternatives - 1	! loop on alternatives


	    found = .FALSE.
	    natt = 0

*	      loop until a good alternative is found

	      DO WHILE (.NOT. found)

*	      generate tentative answer
C Changed by isidoro.ferrante@pi.infn.it 28/3/2000
C to generate within a better range
              if (solution(ique).ne.0.) then
                exponent= ll+ran1 (iseed) *(ul-ll)
                attempt = solution(ique)*(10**exponent)
              else
	        attempt = alt_low  +  ran1 (iseed) * (alt_up - alt_low)
              endif
c	      attempt = alt_low  +  ran1 (iseed) * (alt_up - alt_low)
	      rndm_sign = SIGN (1., (ran1 (iseed) - 0.5))
	      IF (draw_sign(ique))  attempt = rndm_sign * attempt
	      natt = natt + 1

*	      check that difference with solution and other alternative
*	      answers is greater than user defined parameter gap

              found = .TRUE.
c              if (solution(ique).ne.0.)then
c  	      rel1 = ABS (attempt - solution(ique)) / ABS (solution(ique))
c		else
c	      rel1 = ABS (attempt)
c              endif
c	      rel2 = ABS (attempt - solution(ique)) / ABS (attempt)
c	      found = (rel1 .GT. gap) .AND. (rel2 .GT. gap)
		rel1 = ABS (attempt - solution(ique))
		rel2 = ABS (attempt + solution(ique))
                found = (rel1 .GT. gap*rel2)

	        DO j = 1, ialt-1

c	        rel1 = ABS (attempt - altern(ique, j)) / ABS (altern(ique, j))
c	        rel2 = ABS (attempt - altern(ique, j)) / ABS (attempt)
c	        found = found .AND. (rel1 .GT. gap) .AND. (rel2 .GT. gap)

  		rel1 = ABS (attempt - altern(ique, j))
		rel2 = ABS (attempt + altern(ique, j))
		found = found .AND. (rel1 .GT. gap*rel2)

	        END DO
	        
*	Check that attempt is away from the forced answers
            do j = i0, i1-1
               rel1 = ABS (attempt - forced_a(j))
               rel2 = ABS (attempt + forced_a(j))
               found = found .and. (rel1 .GT. rel2*gap)
            enddo
            
*	        stop if too many attempts

	        IF (natt .GT. max_attempt)  THEN

	          PRINT *, ' '
	          PRINT *, ' GENERATE_ANSWERS: error in "compito"', icomp
	          PRINT *, ' alternative to solution', ique, ' not found'
	          PRINT *, ' value of solution is', solution(ique)
	          PRINT *, ' range of alternatives is', alt_low, alt_up
	          PRINT *, ' exeeded number of attempts:', max_attempt
	          PRINT *, ' '
	          CALL LIBSTOP (%VAL(ss_abort))

	        END IF

	      END DO

*	    store found alternative
c	    altern(ique, ialt) = round (attempt, n_digits)
            altern(ique, ialt) = attempt
        

40000	    CONTINUE				! end of loop on alternatives

*  	substitute i1 - i0 alternatives with corresponding forced answers
* 	First, generate where we must substitute the forced answer
         ialt = 1 + INT ( ran1 (iseed) * (n_alternatives-1))
         
*	then substitute
         do j = i0, i1 - 1
            ia1 = mod(natt+ialt+j,n_alternatives-1)+1
            altern(ique,ia1) = forced_a(j)
            iforced(j) = ia1
         enddo
         
c 	update the internal index
c        i0 = i1
         

c*        check if one or more of the alternatives should be "forced"
c         ialt = 1  +  INT ( ran1 (iseed) * (n_alternatives-1))

c         call forced_answers (ique, ialt)
      
*        force the alternatives to have the required precision
         do ialt = 1, n_alternatives - 1	! loop on alternatives
      
            altern(ique, ialt) = round (altern(ique, ialt), n_digits)
         
         enddo
      

*	  randomly choose the position of the correct solution
	  icorrect(ique) = 1  +  INT ( ran1 (iseed) * n_alternatives)
	  altern(ique, n_alternatives) = altern(ique, icorrect(ique))
	  altern(ique, icorrect(ique)) = round (solution(ique), n_digits)

          do j = i0, i1-1
             if(iforced(j).eq.icorrect(ique))iforced(j)=n_alternatives
             iforced(j) = iforced(j) + ique*n_alternatives
          enddo 
          
C       update the internal index
          i0 = i1

           ELSE

           icorrect(ique) = 19

          ENDIF
50000	  CONTINUE				! end of loop on questions

* set the control variable
      ok = .TRUE.
      n_forced = i1 - 1

	RETURN

	END
